#include <sta/debug/spatz.hpp>

#ifdef STA_SPATZ_ENABLED

#ifndef STA_INIT_SPATZ_ID
#	error "The ID used to communicate SPATZ initialization was not defined."
#endif // STA_INIT_SPATZ_ID

#include <cmsis_os2.h>
#include <FreeRTOS.h>
#include <usart.h>

namespace sta
{
    namespace spatz
    {
        sta::Mutex * mutex_;

        void init(sta::Mutex * mutex)
        {
            mutex_ = mutex;
            uint8_t msg = 0xFF;

            sta::lock_guard<sta::Mutex> lock(*mutex_);

            // Wait until SPATZ sends the init byte.
            while (msg != STA_INIT_SPATZ_ID)
            {
            	Debug->read(&msg, 1);
            }
        }

        void request(uint8_t id, uint8_t * buffer, size_t length)
        {
            sta::lock_guard<sta::Mutex> lock(*mutex_);

			Debug->printf("%d", id);
			Debug->read(buffer, length);
        }

        void request(uint8_t id, float * buffer, size_t length)
        {
            mutex_->acquire();

            bool success = false;

            while (!success)
            {
            	Debug->printf("%d", id);

            	HAL_StatusTypeDef state = HAL_UART_Receive(&huart1, reinterpret_cast<uint8_t*>(buffer), length * sizeof(float), 100);

				if (state == HAL_TIMEOUT)
				{
					success = false;
					Debug->println(">ERROR");
					break;
				}

				success = true;
            }

            mutex_->release();
        }

        void request(uint8_t id, double * buffer, size_t length)
        {
            sta::lock_guard<sta::Mutex> lock(*mutex_);

			Debug->printf("%d", id);
			Debug->read(buffer, length);
        }
    } // namespace spatz    
} // namespace sta

#else

namespace sta
{
	namespace spatz
	{
		void init(sta::Mutex * mutex)
	    {
	        // Nothing to do here.
	    }
	}  // namespace spatz
} // namespace sta


#endif // STA_SPATZ_ENABLED
